smoke

package
v0.3.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 2, 2026 License: Apache-2.0 Imports: 23 Imported by: 0

Documentation

Overview

Package smoke runs a smoke test to generate a scoreboard.Scenario.

Index

Constants

const ContentsImage = `` /* 1101-byte string literal not displayed */

ContentsImage is the prompt for image generation.

Variables

This section is empty.

Functions

func Run

Run runs a smoke test on the provider and model.

It returns the supported Scenario as found in practice.

ProviderFactory must be concurrent safe and never fail.

Types

type ProviderFactory

type ProviderFactory func(name string) genai.Provider

ProviderFactory is a function that returns a provider instance. The name represents the sub-test name.

This may be used for HTTP recording and replays.

Directories

Path Synopsis
Package smoketest runs a scoreboard in test mode.
Package smoketest runs a scoreboard in test mode.

Jump to