Documentation
¶
Overview ¶
Package smoke runs a smoke test to generate a scoreboard.Scenario.
Index ¶
Constants ¶
const ContentsImage = `` /* 1101-byte string literal not displayed */
ContentsImage is the prompt for image generation.
Variables ¶
This section is empty.
Functions ¶
func Run ¶
func Run(ctx context.Context, pf ProviderFactory) (scoreboard.Scenario, genai.Usage, error)
Run runs a smoke test on the provider and model.
It returns the supported Scenario as found in practice.
ProviderFactory must be concurrent safe and never fail.
Types ¶
type ProviderFactory ¶
ProviderFactory is a function that returns a provider instance. The name represents the sub-test name.
This may be used for HTTP recording and replays.
Click to show internal directories.
Click to hide internal directories.